@@ -3,6 +3,7 @@ pub mod output;
 pub mod panes;
 pub mod tab;
 
+mod background_jobs;
 mod logging_pipe;
 mod plugins;
 mod pty;
@@ -13,6 +14,7 @@ mod terminal_bytes;
 mod thread_bus;
 mod ui;
 
+use background_jobs::{background_jobs_main, BackgroundJob};
 use log::info;
 use pty_writer::{pty_writer_main, PtyWriteInstruction};
 use std::collections::{HashMap, HashSet};
@@ -40,7 +42,7 @@ use zellij_utils::{
     cli::CliArgs,
     consts::{DEFAULT_SCROLL_BUFFER_SIZE, SCROLL_BUFFER_SIZE},
     data::{Event, PluginCapabilities},
-    errors::{ContextType, ErrorInstruction, FatalError, ServerContext},
+    errors::{prelude::*, ContextType, ErrorInstruction, FatalError, ServerContext},
     input::{
         command::{RunCommand, TerminalAction},
         get_mode_info,
@@ -75,6 +77,7 @@ pub enum ServerInstruction {
     AttachClient(ClientAttributes, Options, ClientId),
     ConnStatus(ClientId),
     ActiveClients(ClientId),
+    Log(Vec<String>, ClientId),
 }
 
 impl From<&ServerInstruction> for ServerContext {
@@ -91,6 +94,7 @@ impl From<&ServerInstruction> for ServerContext {
             ServerInstruction::AttachClient(..) => ServerContext::AttachClient,
             ServerInstruction::ConnStatus(..) => ServerContext::ConnStatus,
             ServerInstruction::ActiveClients(_) => ServerContext::ActiveClients,
+            ServerInstruction::Log(..) => ServerContext::Log,
         }
     }
 }
@@ -106,10 +110,12 @@ pub(crate) struct SessionMetaData {
     pub capabilities: PluginCapabilities,
     pub client_attributes: ClientAttributes,
     pub default_shell: Option<TerminalAction>,
+    pub layout: Box<Layout>,
     screen_thread: Option<thread::JoinHandle<()>>,
     pty_thread: Option<thread::JoinHandle<()>>,
     plugin_thread: Option<thread::JoinHandle<()>>,
     pty_writer_thread: Option<thread::JoinHandle<()>>,
+    background_jobs_thread: Option<thread::JoinHandle<()>>,
 }
 
 impl Drop for SessionMetaData {
@@ -118,6 +124,7 @@ impl Drop for SessionMetaData {
         let _ = self.senders.send_to_screen(ScreenInstruction::Exit);
         let _ = self.senders.send_to_plugin(PluginInstruction::Exit);
         let _ = self.senders.send_to_pty_writer(PtyWriteInstruction::Exit);
+        let _ = self.senders.send_to_background_jobs(BackgroundJob::Exit);
         if let Some(screen_thread) = self.screen_thread.take() {
             let _ = screen_thread.join();
         }
@@ -130,6 +137,9 @@ impl Drop for SessionMetaData {
         if let Some(pty_writer_thread) = self.pty_writer_thread.take() {
             let _ = pty_writer_thread.join();
         }
+        if let Some(background_jobs_thread) = self.background_jobs_thread.take() {
+            let _ = background_jobs_thread.join();
+        }
     }
 }
 
@@ -143,7 +153,21 @@ macro_rules! remove_client {
 macro_rules! send_to_client {
     ($client_id:expr, $os_input:expr, $msg:expr, $session_state:expr) => {
         let send_to_client_res = $os_input.send_to_client($client_id, $msg);
-        if let Err(_) = send_to_client_res {
+        if let Err(e) = send_to_client_res {
+            // Try to recover the message
+            let context = match e.downcast_ref::<ZellijError>() {
+                Some(ZellijError::ClientTooSlow { .. }) => {
+                    format!(
+                        "client {} is processing server messages too slow",
+                        $client_id
+                    )
+                },
+                _ => {
+                    format!("failed to route server message to client {}", $client_id)
+                },
+            };
+            // Log it so it isn't lost
+            Err::<(), _>(e).context(context).non_fatal();
             // failed to send to client, remove it
             remove_client!($client_id, $os_input, $session_state);
         }
@@ -325,7 +349,7 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     })
                 });
 
-                let spawn_tabs = |tab_layout, tab_name| {
+                let spawn_tabs = |tab_layout, floating_panes_layout, tab_name, swap_layouts| {
                     session_data
                         .read()
                         .unwrap()
@@ -335,15 +359,25 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                         .send_to_screen(ScreenInstruction::NewTab(
                             default_shell.clone(),
                             tab_layout,
+                            floating_panes_layout,
                             tab_name,
+                            swap_layouts,
                             client_id,
                         ))
                         .unwrap()
                 };
 
                 if layout.has_tabs() {
-                    for (tab_name, tab_layout) in layout.tabs() {
-                        spawn_tabs(Some(tab_layout.clone()), tab_name);
+                    for (tab_name, tab_layout, floating_panes_layout) in layout.tabs() {
+                        spawn_tabs(
+                            Some(tab_layout.clone()),
+                            floating_panes_layout.clone(),
+                            tab_name,
+                            (
+                                layout.swap_tiled_layouts.clone(),
+                                layout.swap_floating_layouts.clone(),
+                            ),
+                        );
                     }
 
                     if let Some(focused_tab_index) = layout.focused_tab_index() {
@@ -353,14 +387,22 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                             .as_ref()
                             .unwrap()
                             .senders
-                            .send_to_pty(PtyInstruction::GoToTab(
+                            .send_to_screen(ScreenInstruction::GoToTab(
                                 (focused_tab_index + 1) as u32,
-                                client_id,
+                                Some(client_id),
                             ))
                             .unwrap();
                     }
                 } else {
-                    spawn_tabs(None, None);
+                    spawn_tabs(
+                        None,
+                        layout.template.map(|t| t.1).clone().unwrap_or_default(),
+                        None,
+                        (
+                            layout.swap_tiled_layouts.clone(),
+                            layout.swap_floating_layouts.clone(),
+                        ),
+                    );
                 }
                 session_data
                     .read()
@@ -540,6 +582,9 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                 // If `None`- Send an exit instruction. This is the case when a user closes the last Tab/Pane.
                 if let Some(output) = &serialized_output {
                     for (client_id, client_render_instruction) in output.iter() {
+                        // TODO: When a client is too slow or unresponsive, the channel fills up
+                        // and this call will disconnect the client in turn. Should this be
+                        // changed?
                         send_to_client!(
                             *client_id,
                             os_input,
@@ -580,6 +625,14 @@ pub fn start_server(mut os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     session_state
                 );
             },
+            ServerInstruction::Log(lines_to_log, client_id) => {
+                send_to_client!(
+                    client_id,
+                    os_input,
+                    ServerToClientMsg::Log(lines_to_log),
+                    session_state
+                );
+            },
         }
     }
 
@@ -638,6 +691,10 @@ fn init_session(
         channels::unbounded();
     let to_pty_writer = SenderWithContext::new(to_pty_writer);
 
+    let (to_background_jobs, background_jobs_receiver): ChannelWithContext<BackgroundJob> =
+        channels::unbounded();
+    let to_background_jobs = SenderWithContext::new(to_background_jobs);
+
     // Determine and initialize the data directory
     let data_dir = opts.data_dir.unwrap_or_else(get_default_data_dir);
 
@@ -664,6 +721,7 @@ fn init_session(
                     Some(&to_plugin),
                     Some(&to_server),
                     Some(&to_pty_writer),
+                    Some(&to_background_jobs),
                     Some(os_input.clone()),
                 ),
                 opts.debug,
@@ -684,6 +742,7 @@ fn init_session(
                 Some(&to_plugin),
                 Some(&to_server),
                 Some(&to_pty_writer),
+                Some(&to_background_jobs),
                 Some(os_input.clone()),
             );
             let max_panes = opts.max_panes;
@@ -711,10 +770,12 @@ fn init_session(
                 Some(&to_plugin),
                 None,
                 Some(&to_pty_writer),
+                Some(&to_background_jobs),
                 None,
             );
-            let store = Store::default();
+            let store = get_store();
 
+            let layout = layout.clone();
             move || {
                 plugin_thread_main(
                     plugin_bus,
@@ -738,27 +799,60 @@ fn init_session(
                 Some(&to_plugin),
                 Some(&to_server),
                 None,
+                Some(&to_background_jobs),
                 Some(os_input.clone()),
             );
             || pty_writer_main(pty_writer_bus).fatal()
         })
         .unwrap();
 
+    let background_jobs_thread = thread::Builder::new()
+        .name("background_jobs".to_string())
+        .spawn({
+            let background_jobs_bus = Bus::new(
+                vec![background_jobs_receiver],
+                Some(&to_screen),
+                Some(&to_pty),
+                Some(&to_plugin),
+                Some(&to_server),
+                Some(&to_pty_writer),
+                None,
+                Some(os_input.clone()),
+            );
+            || background_jobs_main(background_jobs_bus).fatal()
+        })
+        .unwrap();
+
     SessionMetaData {
         senders: ThreadSenders {
             to_screen: Some(to_screen),
             to_pty: Some(to_pty),
             to_plugin: Some(to_plugin),
             to_pty_writer: Some(to_pty_writer),
+            to_background_jobs: Some(to_background_jobs),
             to_server: None,
             should_silently_fail: false,
         },
         capabilities,
         default_shell,
         client_attributes,
+        layout,
         screen_thread: Some(screen_thread),
         pty_thread: Some(pty_thread),
         plugin_thread: Some(plugin_thread),
         pty_writer_thread: Some(pty_writer_thread),
+        background_jobs_thread: Some(background_jobs_thread),
     }
 }
+
+#[cfg(not(feature = "singlepass"))]
+fn get_store() -> Store {
+    log::info!("Compiling plugins using Cranelift");
+    Store::new(&wasmer::Universal::new(wasmer::Cranelift::default()).engine())
+}
+
+#[cfg(feature = "singlepass")]
+fn get_store() -> Store {
+    log::info!("Compiling plugins using Singlepass");
+    Store::new(&wasmer::Universal::new(wasmer::Singlepass::default()).engine())
+}
