@@ -25,7 +25,8 @@ use zellij_utils::{
     data::{ModeInfo, Style},
     errors::prelude::*,
     input::command::RunCommand,
-    pane_size::{Offset, PaneGeom, Size, Viewport},
+    input::layout::FloatingPaneLayout,
+    pane_size::{Dimension, Offset, PaneGeom, Size, SizeInPixels, Viewport},
 };
 
 const RESIZE_INCREMENT_WIDTH: usize = 5;
@@ -38,6 +39,7 @@ pub struct FloatingPanes {
     connected_clients: Rc<RefCell<HashSet<ClientId>>>,
     connected_clients_in_app: Rc<RefCell<HashSet<ClientId>>>,
     mode_info: Rc<RefCell<HashMap<ClientId, ModeInfo>>>,
+    character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
     default_mode_info: ModeInfo,
     style: Style,
     session_is_mirrored: bool,
@@ -58,6 +60,7 @@ impl FloatingPanes {
         connected_clients: Rc<RefCell<HashSet<ClientId>>>,
         connected_clients_in_app: Rc<RefCell<HashSet<ClientId>>>,
         mode_info: Rc<RefCell<HashMap<ClientId, ModeInfo>>>,
+        character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
         session_is_mirrored: bool,
         default_mode_info: ModeInfo,
         style: Style,
@@ -71,6 +74,7 @@ impl FloatingPanes {
             connected_clients,
             connected_clients_in_app,
             mode_info,
+            character_cell_size,
             session_is_mirrored,
             default_mode_info,
             style,
@@ -199,6 +203,14 @@ impl FloatingPanes {
     pub fn active_pane_id(&self, client_id: ClientId) -> Option<PaneId> {
         self.active_panes.get(&client_id).copied()
     }
+    pub fn active_pane_id_or_focused_pane_id(&self, client_id: Option<ClientId>) -> Option<PaneId> {
+        // returns the focused pane of any client_id - should be safe because the way things are
+        // set up at the time of writing, all clients are focused on the same floating pane due to
+        // z_index issues
+        client_id
+            .and_then(|client_id| self.active_panes.get(&client_id).copied())
+            .or_else(|| self.panes.keys().next().copied())
+    }
     pub fn toggle_show_panes(&mut self, should_show_floating_panes: bool) {
         self.show_panes = should_show_floating_panes;
         if should_show_floating_panes {
@@ -224,9 +236,55 @@ impl FloatingPanes {
         );
         floating_pane_grid.find_room_for_new_pane()
     }
+    pub fn position_floating_pane_layout(
+        &mut self,
+        floating_pane_layout: &FloatingPaneLayout,
+    ) -> PaneGeom {
+        let display_area = *self.display_area.borrow();
+        let viewport = *self.viewport.borrow();
+        let floating_pane_grid = FloatingPaneGrid::new(
+            &mut self.panes,
+            &mut self.desired_pane_positions,
+            display_area,
+            viewport,
+        );
+        let mut position = floating_pane_grid.find_room_for_new_pane().unwrap(); // TODO: no unwrap
+        if let Some(x) = &floating_pane_layout.x {
+            position.x = x.to_position(viewport.cols);
+        }
+        if let Some(y) = &floating_pane_layout.y {
+            position.y = y.to_position(viewport.rows);
+        }
+        if let Some(width) = &floating_pane_layout.width {
+            position.cols = Dimension::fixed(width.to_position(viewport.cols));
+        }
+        if let Some(height) = &floating_pane_layout.height {
+            position.rows = Dimension::fixed(height.to_position(viewport.rows));
+        }
+        if position.cols.as_usize() > viewport.cols {
+            position.cols = Dimension::fixed(viewport.cols);
+        }
+        if position.rows.as_usize() > viewport.rows {
+            position.rows = Dimension::fixed(viewport.rows);
+        }
+        if position.x + position.cols.as_usize() > viewport.cols {
+            position.x = position
+                .x
+                .saturating_sub((position.x + position.cols.as_usize()) - viewport.cols);
+        }
+        if position.y + position.rows.as_usize() > viewport.rows {
+            position.y = position
+                .y
+                .saturating_sub((position.y + position.rows.as_usize()) - viewport.rows);
+        }
+        position
+    }
     pub fn first_floating_pane_id(&self) -> Option<PaneId> {
         self.panes.keys().next().copied()
     }
+    pub fn last_floating_pane_id(&self) -> Option<PaneId> {
+        self.panes.keys().last().copied()
+    }
     pub fn first_active_floating_pane_id(&self) -> Option<PaneId> {
         self.active_panes.values().next().copied()
     }
@@ -249,7 +307,8 @@ impl FloatingPanes {
             } else {
                 pane.set_content_offset(Offset::default());
             }
-            resize_pty!(pane, os_api, self.senders).with_context(|| err_context(&pane.pid()))?;
+            resize_pty!(pane, os_api, self.senders, self.character_cell_size)
+                .with_context(|| err_context(&pane.pid()))?;
         }
         Ok(())
     }
@@ -286,6 +345,9 @@ impl FloatingPanes {
                 &active_panes,
                 multiple_users_exist_in_session,
                 Some(z_index + 1), // +1 because 0 is reserved for non-floating panes
+                false,
+                false,
+                true,
             );
             for client_id in &connected_clients {
                 let client_mode = self
@@ -332,7 +394,7 @@ impl FloatingPanes {
 
     pub fn resize_pty_all_panes(&mut self, os_api: &mut Box<dyn ServerOsApi>) -> Result<()> {
         for pane in self.panes.values_mut() {
-            resize_pty!(pane, os_api, self.senders)
+            resize_pty!(pane, os_api, self.senders, self.character_cell_size)
                 .with_context(|| format!("failed to resize PTY in pane {:?}", pane.pid()))?;
         }
         Ok(())
@@ -366,7 +428,8 @@ impl FloatingPanes {
                 .with_context(err_context)?;
 
             for pane in self.panes.values_mut() {
-                resize_pty!(pane, os_api, self.senders).with_context(err_context)?;
+                resize_pty!(pane, os_api, self.senders, self.character_cell_size)
+                    .with_context(err_context)?;
             }
             self.set_force_render();
             return Ok(true);
@@ -523,6 +586,50 @@ impl FloatingPanes {
             self.set_force_render();
         }
     }
+    pub fn move_active_pane(
+        &mut self,
+        search_backwards: bool,
+        os_api: &mut Box<dyn ServerOsApi>,
+        client_id: ClientId,
+    ) {
+        let active_pane_id = self.get_active_pane_id(client_id).unwrap();
+
+        let new_position_id = {
+            let pane_grid = FloatingPaneGrid::new(
+                &mut self.panes,
+                &mut self.desired_pane_positions,
+                *self.display_area.borrow(),
+                *self.viewport.borrow(),
+            );
+            if search_backwards {
+                pane_grid.previous_selectable_pane_id(&active_pane_id)
+            } else {
+                pane_grid.next_selectable_pane_id(&active_pane_id)
+            }
+        };
+        if let Some(new_position_id) = new_position_id {
+            let current_position = self.panes.get(&active_pane_id).unwrap();
+            let prev_geom = current_position.position_and_size();
+            let prev_geom_override = current_position.geom_override();
+
+            let new_position = self.panes.get_mut(&new_position_id).unwrap();
+            let next_geom = new_position.position_and_size();
+            let next_geom_override = new_position.geom_override();
+            new_position.set_geom(prev_geom);
+            if let Some(geom) = prev_geom_override {
+                new_position.set_geom_override(geom);
+            }
+            new_position.set_should_render(true);
+
+            let current_position = self.panes.get_mut(&active_pane_id).unwrap();
+            current_position.set_geom(next_geom);
+            if let Some(geom) = next_geom_override {
+                current_position.set_geom_override(geom);
+            }
+            current_position.set_should_render(true);
+            let _ = self.set_pane_frames(os_api);
+        }
+    }
     pub fn move_clients_out_of_pane(&mut self, pane_id: PaneId) {
         let active_panes: Vec<(ClientId, PaneId)> = self
             .active_panes
@@ -688,6 +795,17 @@ impl FloatingPanes {
     pub fn get_panes(&self) -> impl Iterator<Item = (&PaneId, &Box<dyn Pane>)> {
         self.panes.iter()
     }
+    pub fn visible_panes_count(&self) -> usize {
+        self.panes.len()
+    }
+    pub fn drain(&mut self) -> BTreeMap<PaneId, Box<dyn Pane>> {
+        self.z_indices.clear();
+        self.desired_pane_positions.clear();
+        match self.panes.iter().next().map(|(pid, _p)| *pid) {
+            Some(first_pid) => self.panes.split_off(&first_pid),
+            None => BTreeMap::new(),
+        }
+    }
     fn move_clients_between_panes(&mut self, from_pane_id: PaneId, to_pane_id: PaneId) {
         let clients_in_pane: Vec<ClientId> = self
             .active_panes
@@ -701,4 +819,42 @@ impl FloatingPanes {
                 .insert(client_id, to_pane_id, &mut self.panes);
         }
     }
+    pub fn reapply_pane_focus(&mut self) {
+        if let Some(focused_pane) = self.first_active_floating_pane_id() {
+            // floating pane focus is the same for all clients
+            self.focus_pane_for_all_clients(focused_pane);
+        }
+    }
+    pub fn switch_active_pane_with(&mut self, os_api: &mut Box<dyn ServerOsApi>, pane_id: PaneId) {
+        if let Some(active_pane_id) = self.first_active_floating_pane_id() {
+            let current_position = self.panes.get(&active_pane_id).unwrap();
+            let prev_geom = current_position.position_and_size();
+            let prev_geom_override = current_position.geom_override();
+
+            let new_position = self.panes.get_mut(&pane_id).unwrap();
+            let next_geom = new_position.position_and_size();
+            let next_geom_override = new_position.geom_override();
+            new_position.set_geom(prev_geom);
+            if let Some(geom) = prev_geom_override {
+                new_position.set_geom_override(geom);
+            }
+            resize_pty!(new_position, os_api, self.senders, self.character_cell_size).unwrap();
+            new_position.set_should_render(true);
+
+            let current_position = self.panes.get_mut(&active_pane_id).unwrap();
+            current_position.set_geom(next_geom);
+            if let Some(geom) = next_geom_override {
+                current_position.set_geom_override(geom);
+            }
+            resize_pty!(
+                current_position,
+                os_api,
+                self.senders,
+                self.character_cell_size
+            )
+            .unwrap();
+            current_position.set_should_render(true);
+            self.focus_pane_for_all_clients(active_pane_id);
+        }
+    }
 }
