@@ -16,7 +16,8 @@ use zellij_utils::{
     channels::SenderWithContext,
     data::{Event, InputMode, Mouse, Palette, PaletteColor, Style},
     errors::prelude::*,
-    pane_size::{Dimension, PaneGeom},
+    input::layout::Run,
+    pane_size::PaneGeom,
     shared::make_terminal_title,
     vte,
 };
@@ -64,6 +65,8 @@ pub(crate) struct PluginPane {
     prev_pane_name: String,
     frame: HashMap<ClientId, PaneFrame>,
     borderless: bool,
+    pane_frame_color_override: Option<(PaletteColor, Option<String>)>,
+    invoked_with: Option<Run>,
 }
 
 impl PluginPane {
@@ -79,6 +82,7 @@ impl PluginPane {
         link_handler: Rc<RefCell<LinkHandler>>,
         character_cell_size: Rc<RefCell<Option<SizeInPixels>>>,
         style: Style,
+        invoked_with: Option<Run>,
     ) -> Self {
         Self {
             pid,
@@ -102,6 +106,8 @@ impl PluginPane {
             vte_parsers: HashMap::new(),
             grids: HashMap::new(),
             style,
+            pane_frame_color_override: None,
+            invoked_with,
         }
     }
 }
@@ -220,6 +226,11 @@ impl Pane for PluginPane {
             if self.should_render.get(&client_id).copied().unwrap_or(false) {
                 let content_x = self.get_content_x();
                 let content_y = self.get_content_y();
+                let rows = self.get_content_rows();
+                let columns = self.get_content_columns();
+                if rows < 1 || columns < 1 {
+                    return Ok(None);
+                }
                 if let Some(grid) = self.grids.get_mut(&client_id) {
                     match grid.render(content_x, content_y, &self.style) {
                         Ok(rendered_assets) => {
@@ -244,7 +255,13 @@ impl Pane for PluginPane {
         }
         if let Some(grid) = self.grids.get(&client_id) {
             let err_context = || format!("failed to render frame for client {client_id}");
-            let pane_title = if self.pane_name.is_empty()
+            let pane_title = if let Some(text_color_override) = self
+                .pane_frame_color_override
+                .as_ref()
+                .and_then(|(_color, text)| text.as_ref())
+            {
+                text_color_override.into()
+            } else if self.pane_name.is_empty()
                 && input_mode == InputMode::RenamePane
                 && frame_params.is_main_client
             {
@@ -257,12 +274,22 @@ impl Pane for PluginPane {
                 self.pane_name.clone()
             };
 
-            let frame = PaneFrame::new(
-                self.current_geom().into(),
+            let mut frame_geom = self.current_geom();
+            if !frame_params.should_draw_pane_frames {
+                // in this case the width of the frame needs not include the pane corners
+                frame_geom
+                    .cols
+                    .set_inner(frame_geom.cols.as_usize().saturating_sub(1));
+            }
+            let mut frame = PaneFrame::new(
+                frame_geom.into(),
                 grid.scrollback_position_and_length(),
                 pane_title,
                 frame_params,
             );
+            if let Some((frame_color_override, _text)) = self.pane_frame_color_override.as_ref() {
+                frame.override_color(*frame_color_override);
+            }
 
             let res = match self.frame.get(&client_id) {
                 // TODO: use and_then or something?
@@ -330,28 +357,28 @@ impl Pane for PluginPane {
     }
     fn reduce_height(&mut self, percent: f64) {
         if let Some(p) = self.geom.rows.as_percent() {
-            self.geom.rows = Dimension::percent(p - percent);
+            self.geom.rows.set_percent(p - percent);
             self.resize_grids();
             self.set_should_render(true);
         }
     }
     fn increase_height(&mut self, percent: f64) {
         if let Some(p) = self.geom.rows.as_percent() {
-            self.geom.rows = Dimension::percent(p + percent);
+            self.geom.rows.set_percent(p + percent);
             self.resize_grids();
             self.set_should_render(true);
         }
     }
     fn reduce_width(&mut self, percent: f64) {
         if let Some(p) = self.geom.cols.as_percent() {
-            self.geom.cols = Dimension::percent(p - percent);
+            self.geom.cols.set_percent(p - percent);
             self.resize_grids();
             self.set_should_render(true);
         }
     }
     fn increase_width(&mut self, percent: f64) {
         if let Some(p) = self.geom.cols.as_percent() {
-            self.geom.cols = Dimension::percent(p + percent);
+            self.geom.cols.set_percent(p + percent);
             self.resize_grids();
             self.set_should_render(true);
         }
@@ -469,6 +496,23 @@ impl Pane for PluginPane {
             )]))
             .unwrap();
     }
+    fn add_red_pane_frame_color_override(&mut self, error_text: Option<String>) {
+        self.pane_frame_color_override = Some((self.style.colors.red, error_text));
+    }
+    fn clear_pane_frame_color_override(&mut self) {
+        self.pane_frame_color_override = None;
+    }
+    fn frame_color_override(&self) -> Option<PaletteColor> {
+        self.pane_frame_color_override
+            .as_ref()
+            .map(|(color, _text)| *color)
+    }
+    fn invoked_with(&self) -> &Option<Run> {
+        &self.invoked_with
+    }
+    fn set_title(&mut self, title: String) {
+        self.pane_title = title;
+    }
 }
 
 impl PluginPane {
