@@ -15,7 +15,7 @@ use zellij_utils::{
     errors::{ContextType, PtyContext},
     input::{
         command::{RunCommand, TerminalAction},
-        layout::{Layout, PaneLayout, Run, RunPluginLocation},
+        layout::{FloatingPaneLayout, Layout, Run, RunPluginLocation, TiledPaneLayout},
     },
 };
 
@@ -49,8 +49,8 @@ pub enum PtyInstruction {
     GoToTab(TabIndex, ClientId),
     NewTab(
         Option<TerminalAction>,
-        Option<PaneLayout>,
-        Option<String>,
+        Option<TiledPaneLayout>,
+        Vec<FloatingPaneLayout>,
         usize,                                // tab_index
         HashMap<RunPluginLocation, Vec<u32>>, // plugin_ids
         ClientId,
@@ -146,6 +146,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                         pty.bus.senders.clone(),
                                         *terminal_id,
                                         run_command.clone(),
+                                        None,
                                     )
                                     .with_context(err_context)?;
                                 }
@@ -164,7 +165,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                     || format!("failed to open in-place editor for client {}", client_id);
 
                 match pty.spawn_terminal(
-                    Some(TerminalAction::OpenFile(temp_file, line_number)),
+                    Some(TerminalAction::OpenFile(temp_file, line_number, None)),
                     ClientOrTabIndex::ClientId(client_id),
                 ) {
                     Ok((pid, _starts_held)) => {
@@ -242,6 +243,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                             Some(2), // exit status
                                             run_command,
                                             None,
+                                            None,
                                         ))
                                         .with_context(err_context)?;
                                 }
@@ -312,6 +314,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                             Some(2), // exit status
                                             run_command,
                                             None,
+                                            None,
                                         ))
                                         .with_context(err_context)?;
                                 }
@@ -335,36 +338,27 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
             PtyInstruction::NewTab(
                 terminal_action,
                 tab_layout,
-                tab_name,
+                floating_panes_layout,
                 tab_index,
                 plugin_ids,
                 client_id,
             ) => {
                 let err_context = || format!("failed to open new tab for client {}", client_id);
 
+                let floating_panes_layout = if floating_panes_layout.is_empty() {
+                    layout.new_tab().1
+                } else {
+                    floating_panes_layout
+                };
                 pty.spawn_terminals_for_layout(
-                    tab_layout.unwrap_or_else(|| layout.new_tab()),
+                    tab_layout.unwrap_or_else(|| layout.new_tab().0),
+                    floating_panes_layout,
                     terminal_action.clone(),
                     plugin_ids,
                     tab_index,
                     client_id,
                 )
                 .with_context(err_context)?;
-
-                if let Some(tab_name) = tab_name {
-                    // clear current name at first
-                    pty.bus
-                        .senders
-                        .send_to_screen(ScreenInstruction::UpdateTabName(vec![0], client_id))
-                        .with_context(err_context)?;
-                    pty.bus
-                        .senders
-                        .send_to_screen(ScreenInstruction::UpdateTabName(
-                            tab_name.into_bytes(),
-                            client_id,
-                        ))
-                        .with_context(err_context)?;
-                }
             },
             PtyInstruction::ClosePane(id) => {
                 pty.close_pane(id)
@@ -414,6 +408,7 @@ pub(crate) fn pty_thread_main(mut pty: Pty, layout: Box<Layout>) -> Result<()> {
                                         Some(2), // exit status
                                         run_command,
                                         None,
+                                        None,
                                     ))
                                     .with_context(err_context)?;
                             }
@@ -443,18 +438,46 @@ impl Pty {
             default_editor,
         }
     }
-    pub fn get_default_terminal(&self, cwd: Option<PathBuf>) -> TerminalAction {
-        let shell = PathBuf::from(env::var("SHELL").unwrap_or_else(|_| {
-            log::warn!("Cannot read SHELL env, falling back to use /bin/sh");
-            "/bin/sh".to_string()
-        }));
-        TerminalAction::RunCommand(RunCommand {
-            args: vec![],
-            command: shell,
-            cwd, // note: this might also be filled by the calling function, eg. spawn_terminal
-            hold_on_close: false,
-            hold_on_start: false,
-        })
+    pub fn get_default_terminal(
+        &self,
+        cwd: Option<PathBuf>,
+        default_shell: Option<TerminalAction>,
+    ) -> TerminalAction {
+        match default_shell {
+            Some(mut default_shell) => {
+                if let Some(cwd) = cwd {
+                    match default_shell {
+                        TerminalAction::RunCommand(ref mut command) => {
+                            command.cwd = Some(cwd);
+                        },
+                        TerminalAction::OpenFile(ref _file, _line_number, ref mut edit_cwd) => {
+                            match edit_cwd.as_mut() {
+                                Some(edit_cwd) => {
+                                    *edit_cwd = cwd.join(&edit_cwd);
+                                },
+                                None => {
+                                    let _ = edit_cwd.insert(cwd.clone());
+                                },
+                            };
+                        },
+                    }
+                }
+                default_shell
+            },
+            None => {
+                let shell = PathBuf::from(env::var("SHELL").unwrap_or_else(|_| {
+                    log::warn!("Cannot read SHELL env, falling back to use /bin/sh");
+                    "/bin/sh".to_string()
+                }));
+                TerminalAction::RunCommand(RunCommand {
+                    args: vec![],
+                    command: shell,
+                    cwd, // note: this might also be filled by the calling function, eg. spawn_terminal
+                    hold_on_close: false,
+                    hold_on_start: false,
+                })
+            },
+        }
     }
     fn fill_cwd(&self, terminal_action: &mut TerminalAction, client_id: ClientId) {
         if let TerminalAction::RunCommand(run_command) = terminal_action {
@@ -487,12 +510,12 @@ impl Pty {
         let terminal_action = match client_or_tab_index {
             ClientOrTabIndex::ClientId(client_id) => {
                 let mut terminal_action =
-                    terminal_action.unwrap_or_else(|| self.get_default_terminal(None));
+                    terminal_action.unwrap_or_else(|| self.get_default_terminal(None, None));
                 self.fill_cwd(&mut terminal_action, client_id);
                 terminal_action
             },
             ClientOrTabIndex::TabIndex(_) => {
-                terminal_action.unwrap_or_else(|| self.get_default_terminal(None))
+                terminal_action.unwrap_or_else(|| self.get_default_terminal(None, None))
             },
         };
         let (hold_on_start, hold_on_close) = match &terminal_action {
@@ -505,7 +528,13 @@ impl Pty {
         if hold_on_start {
             // we don't actually open a terminal in this case, just wait for the user to run it
             let starts_held = hold_on_start;
-            let terminal_id = self.bus.os_input.as_mut().unwrap().reserve_terminal_id()?;
+            let terminal_id = self
+                .bus
+                .os_input
+                .as_mut()
+                .context("couldn't get mutable reference to OS interface")
+                .and_then(|os_input| os_input.reserve_terminal_id())
+                .with_context(err_context)?;
             return Ok((terminal_id, starts_held));
         }
 
@@ -518,6 +547,7 @@ impl Pty {
                         exit_status,
                         command,
                         None,
+                        None,
                     ));
                 } else {
                     let _ = senders.send_to_screen(ScreenInstruction::ClosePane(pane_id, None));
@@ -561,7 +591,8 @@ impl Pty {
     }
     pub fn spawn_terminals_for_layout(
         &mut self,
-        layout: PaneLayout,
+        layout: TiledPaneLayout,
+        floating_panes_layout: Vec<FloatingPaneLayout>,
         default_shell: Option<TerminalAction>,
         plugin_ids: HashMap<RunPluginLocation, Vec<u32>>,
         tab_index: usize,
@@ -569,179 +600,32 @@ impl Pty {
     ) -> Result<()> {
         let err_context = || format!("failed to spawn terminals for layout for client {client_id}");
 
-        let mut default_shell = default_shell.unwrap_or_else(|| self.get_default_terminal(None));
+        let mut default_shell =
+            default_shell.unwrap_or_else(|| self.get_default_terminal(None, None));
         self.fill_cwd(&mut default_shell, client_id);
         let extracted_run_instructions = layout.extract_run_instructions();
+        let extracted_floating_run_instructions =
+            floating_panes_layout.iter().map(|f| f.run.clone());
         let mut new_pane_pids: Vec<(u32, bool, Option<RunCommand>, Result<RawFd>)> = vec![]; // (terminal_id,
                                                                                              // starts_held,
                                                                                              // run_command,
                                                                                              // file_descriptor)
+        let mut new_floating_panes_pids: Vec<(u32, bool, Option<RunCommand>, Result<RawFd>)> =
+            vec![]; // same
+                    // as
+                    // new_pane_pids
         for run_instruction in extracted_run_instructions {
-            let quit_cb = Box::new({
-                let senders = self.bus.senders.clone();
-                move |pane_id, _exit_status, _command| {
-                    let _ = senders.send_to_screen(ScreenInstruction::ClosePane(pane_id, None));
-                }
-            });
-            match run_instruction {
-                Some(Run::Command(command)) => {
-                    let starts_held = command.hold_on_start;
-                    let hold_on_close = command.hold_on_close;
-                    let quit_cb = Box::new({
-                        let senders = self.bus.senders.clone();
-                        move |pane_id, exit_status, command| {
-                            if hold_on_close {
-                                let _ = senders.send_to_screen(ScreenInstruction::HoldPane(
-                                    pane_id,
-                                    exit_status,
-                                    command,
-                                    None,
-                                ));
-                            } else {
-                                let _ = senders
-                                    .send_to_screen(ScreenInstruction::ClosePane(pane_id, None));
-                            }
-                        }
-                    });
-                    let cmd = TerminalAction::RunCommand(command.clone());
-                    if starts_held {
-                        // we don't actually open a terminal in this case, just wait for the user to run it
-                        match self
-                            .bus
-                            .os_input
-                            .as_mut()
-                            .context("no OS I/O interface found")
-                            .with_context(err_context)?
-                            .reserve_terminal_id()
-                        {
-                            Ok(terminal_id) => {
-                                new_pane_pids.push((
-                                    terminal_id,
-                                    starts_held,
-                                    Some(command.clone()),
-                                    Ok(terminal_id as i32), // this is not actually correct but gets
-                                                            // stripped later
-                                ));
-                            },
-                            Err(e) => Err::<(), _>(e).with_context(err_context).non_fatal(),
-                        }
-                    } else {
-                        match self
-                            .bus
-                            .os_input
-                            .as_mut()
-                            .context("no OS I/O interface found")
-                            .with_context(err_context)?
-                            .spawn_terminal(cmd, quit_cb, self.default_editor.clone())
-                            .with_context(err_context)
-                        {
-                            Ok((terminal_id, pid_primary, child_fd)) => {
-                                self.id_to_child_pid.insert(terminal_id, child_fd);
-                                new_pane_pids.push((
-                                    terminal_id,
-                                    starts_held,
-                                    Some(command.clone()),
-                                    Ok(pid_primary),
-                                ));
-                            },
-                            Err(err) => match err.downcast_ref::<ZellijError>() {
-                                Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
-                                    new_pane_pids.push((
-                                        *terminal_id,
-                                        starts_held,
-                                        Some(command.clone()),
-                                        Err(err),
-                                    ));
-                                },
-                                _ => {
-                                    Err::<(), _>(err).non_fatal();
-                                },
-                            },
-                        }
-                    }
-                },
-                Some(Run::Cwd(cwd)) => {
-                    let starts_held = false; // we do not hold Cwd panes
-                    let shell = self.get_default_terminal(Some(cwd));
-                    match self
-                        .bus
-                        .os_input
-                        .as_mut()
-                        .context("no OS I/O interface found")
-                        .with_context(err_context)?
-                        .spawn_terminal(shell, quit_cb, self.default_editor.clone())
-                        .with_context(err_context)
-                    {
-                        Ok((terminal_id, pid_primary, child_fd)) => {
-                            self.id_to_child_pid.insert(terminal_id, child_fd);
-                            new_pane_pids.push((terminal_id, starts_held, None, Ok(pid_primary)));
-                        },
-                        Err(err) => match err.downcast_ref::<ZellijError>() {
-                            Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
-                                new_pane_pids.push((*terminal_id, starts_held, None, Err(err)));
-                            },
-                            _ => {
-                                Err::<(), _>(err).non_fatal();
-                            },
-                        },
-                    }
-                },
-                Some(Run::EditFile(path_to_file, line_number)) => {
-                    let starts_held = false; // we do not hold edit panes (for now?)
-                    match self
-                        .bus
-                        .os_input
-                        .as_mut()
-                        .context("no OS I/O interface found")
-                        .with_context(err_context)?
-                        .spawn_terminal(
-                            TerminalAction::OpenFile(path_to_file, line_number),
-                            quit_cb,
-                            self.default_editor.clone(),
-                        )
-                        .with_context(err_context)
-                    {
-                        Ok((terminal_id, pid_primary, child_fd)) => {
-                            self.id_to_child_pid.insert(terminal_id, child_fd);
-                            new_pane_pids.push((terminal_id, starts_held, None, Ok(pid_primary)));
-                        },
-                        Err(err) => match err.downcast_ref::<ZellijError>() {
-                            Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
-                                new_pane_pids.push((*terminal_id, starts_held, None, Err(err)));
-                            },
-                            _ => {
-                                Err::<(), _>(err).non_fatal();
-                            },
-                        },
-                    }
-                },
-                None => {
-                    let starts_held = false;
-                    match self
-                        .bus
-                        .os_input
-                        .as_mut()
-                        .context("no OS I/O interface found")
-                        .with_context(err_context)?
-                        .spawn_terminal(default_shell.clone(), quit_cb, self.default_editor.clone())
-                        .with_context(err_context)
-                    {
-                        Ok((terminal_id, pid_primary, child_fd)) => {
-                            self.id_to_child_pid.insert(terminal_id, child_fd);
-                            new_pane_pids.push((terminal_id, starts_held, None, Ok(pid_primary)));
-                        },
-                        Err(err) => match err.downcast_ref::<ZellijError>() {
-                            Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
-                                new_pane_pids.push((*terminal_id, starts_held, None, Err(err)));
-                            },
-                            _ => {
-                                Err::<(), _>(err).non_fatal();
-                            },
-                        },
-                    }
-                },
-                // Investigate moving plugin loading to here.
-                Some(Run::Plugin(_)) => {},
+            if let Some(new_pane_data) =
+                self.apply_run_instruction(run_instruction, default_shell.clone(), tab_index)?
+            {
+                new_pane_pids.push(new_pane_data);
+            }
+        }
+        for run_instruction in extracted_floating_run_instructions {
+            if let Some(new_pane_data) =
+                self.apply_run_instruction(run_instruction, default_shell.clone(), tab_index)?
+            {
+                new_floating_panes_pids.push(new_pane_data);
             }
         }
         // Option<RunCommand> should only be Some if the pane starts held
@@ -755,17 +639,32 @@ impl Pty {
                 }
             })
             .collect();
+        let new_tab_floating_pane_ids: Vec<(u32, Option<RunCommand>)> = new_floating_panes_pids
+            .iter()
+            .map(|(terminal_id, starts_held, run_command, _)| {
+                if *starts_held {
+                    (*terminal_id, run_command.clone())
+                } else {
+                    (*terminal_id, None)
+                }
+            })
+            .collect();
         self.bus
             .senders
             .send_to_screen(ScreenInstruction::ApplyLayout(
                 layout,
+                floating_panes_layout,
                 new_tab_pane_ids,
+                new_tab_floating_pane_ids,
                 plugin_ids,
                 tab_index,
                 client_id,
             ))
             .with_context(err_context)?;
-        for (terminal_id, starts_held, run_command, pid_primary) in new_pane_pids {
+        let mut terminals_to_start = vec![];
+        terminals_to_start.append(&mut new_pane_pids);
+        terminals_to_start.append(&mut new_floating_panes_pids);
+        for (terminal_id, starts_held, run_command, pid_primary) in terminals_to_start {
             if starts_held {
                 // we do not run a command or start listening for bytes on held panes
                 continue;
@@ -804,6 +703,7 @@ impl Pty {
                                 self.bus.senders.clone(),
                                 terminal_id,
                                 run_command.clone(),
+                                Some(tab_index),
                             )
                             .with_context(err_context)?;
                         } else {
@@ -820,6 +720,174 @@ impl Pty {
         }
         Ok(())
     }
+    fn apply_run_instruction(
+        &mut self,
+        run_instruction: Option<Run>,
+        default_shell: TerminalAction,
+        tab_index: usize,
+    ) -> Result<Option<(u32, bool, Option<RunCommand>, Result<i32>)>> {
+        // terminal_id,
+        // starts_held,
+        // command
+        // successfully opened
+        let err_context = || format!("failed to apply run instruction");
+        let quit_cb = Box::new({
+            let senders = self.bus.senders.clone();
+            move |pane_id, _exit_status, _command| {
+                let _ = senders.send_to_screen(ScreenInstruction::ClosePane(pane_id, None));
+            }
+        });
+        match run_instruction {
+            Some(Run::Command(command)) => {
+                let starts_held = command.hold_on_start;
+                let hold_on_close = command.hold_on_close;
+                let quit_cb = Box::new({
+                    let senders = self.bus.senders.clone();
+                    move |pane_id, exit_status, command| {
+                        if hold_on_close {
+                            let _ = senders.send_to_screen(ScreenInstruction::HoldPane(
+                                pane_id,
+                                exit_status,
+                                command,
+                                Some(tab_index),
+                                None,
+                            ));
+                        } else {
+                            let _ =
+                                senders.send_to_screen(ScreenInstruction::ClosePane(pane_id, None));
+                        }
+                    }
+                });
+                let cmd = TerminalAction::RunCommand(command.clone());
+                if starts_held {
+                    // we don't actually open a terminal in this case, just wait for the user to run it
+                    match self
+                        .bus
+                        .os_input
+                        .as_mut()
+                        .context("no OS I/O interface found")
+                        .with_context(err_context)?
+                        .reserve_terminal_id()
+                    {
+                        Ok(terminal_id) => {
+                            Ok(Some((
+                                terminal_id,
+                                starts_held,
+                                Some(command.clone()),
+                                Ok(terminal_id as i32), // this is not actually correct but gets
+                                                        // stripped later
+                            )))
+                        },
+                        Err(e) => Err(e),
+                    }
+                } else {
+                    match self
+                        .bus
+                        .os_input
+                        .as_mut()
+                        .context("no OS I/O interface found")
+                        .with_context(err_context)?
+                        .spawn_terminal(cmd, quit_cb, self.default_editor.clone())
+                        .with_context(err_context)
+                    {
+                        Ok((terminal_id, pid_primary, child_fd)) => {
+                            self.id_to_child_pid.insert(terminal_id, child_fd);
+                            Ok(Some((
+                                terminal_id,
+                                starts_held,
+                                Some(command.clone()),
+                                Ok(pid_primary),
+                            )))
+                        },
+                        Err(err) => {
+                            match err.downcast_ref::<ZellijError>() {
+                                Some(ZellijError::CommandNotFound { terminal_id, .. }) => Ok(Some(
+                                    (*terminal_id, starts_held, Some(command.clone()), Err(err)),
+                                )),
+                                _ => Err(err),
+                            }
+                        },
+                    }
+                }
+            },
+            Some(Run::Cwd(cwd)) => {
+                let starts_held = false; // we do not hold Cwd panes
+                let shell = self.get_default_terminal(Some(cwd), Some(default_shell.clone()));
+                match self
+                    .bus
+                    .os_input
+                    .as_mut()
+                    .context("no OS I/O interface found")
+                    .with_context(err_context)?
+                    .spawn_terminal(shell, quit_cb, self.default_editor.clone())
+                    .with_context(err_context)
+                {
+                    Ok((terminal_id, pid_primary, child_fd)) => {
+                        self.id_to_child_pid.insert(terminal_id, child_fd);
+                        Ok(Some((terminal_id, starts_held, None, Ok(pid_primary))))
+                    },
+                    Err(err) => match err.downcast_ref::<ZellijError>() {
+                        Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
+                            Ok(Some((*terminal_id, starts_held, None, Err(err))))
+                        },
+                        _ => Err(err),
+                    },
+                }
+            },
+            Some(Run::EditFile(path_to_file, line_number, cwd)) => {
+                let starts_held = false; // we do not hold edit panes (for now?)
+                match self
+                    .bus
+                    .os_input
+                    .as_mut()
+                    .context("no OS I/O interface found")
+                    .with_context(err_context)?
+                    .spawn_terminal(
+                        TerminalAction::OpenFile(path_to_file, line_number, cwd),
+                        quit_cb,
+                        self.default_editor.clone(),
+                    )
+                    .with_context(err_context)
+                {
+                    Ok((terminal_id, pid_primary, child_fd)) => {
+                        self.id_to_child_pid.insert(terminal_id, child_fd);
+                        Ok(Some((terminal_id, starts_held, None, Ok(pid_primary))))
+                    },
+                    Err(err) => match err.downcast_ref::<ZellijError>() {
+                        Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
+                            Ok(Some((*terminal_id, starts_held, None, Err(err))))
+                        },
+                        _ => Err(err),
+                    },
+                }
+            },
+            None => {
+                let starts_held = false;
+                match self
+                    .bus
+                    .os_input
+                    .as_mut()
+                    .context("no OS I/O interface found")
+                    .with_context(err_context)?
+                    .spawn_terminal(default_shell.clone(), quit_cb, self.default_editor.clone())
+                    .with_context(err_context)
+                {
+                    Ok((terminal_id, pid_primary, child_fd)) => {
+                        self.id_to_child_pid.insert(terminal_id, child_fd);
+                        Ok(Some((terminal_id, starts_held, None, Ok(pid_primary))))
+                    },
+                    Err(err) => match err.downcast_ref::<ZellijError>() {
+                        Some(ZellijError::CommandNotFound { terminal_id, .. }) => {
+                            Ok(Some((*terminal_id, starts_held, None, Err(err))))
+                        },
+                        _ => Err(err),
+                    },
+                }
+            },
+            // Investigate moving plugin loading to here.
+            Some(Run::Plugin(_)) => Ok(None),
+        }
+    }
     pub fn close_pane(&mut self, id: PaneId) -> Result<()> {
         let err_context = || format!("failed to close for pane {id:?}");
         match id {
@@ -887,6 +955,7 @@ impl Pty {
                                 exit_status,
                                 command,
                                 None,
+                                None,
                             ));
                         } else {
                             let _ =
@@ -948,6 +1017,7 @@ fn send_command_not_found_to_screen(
     senders: ThreadSenders,
     terminal_id: u32,
     run_command: RunCommand,
+    tab_index: Option<usize>,
 ) -> Result<()> {
     let err_context = || format!("failed to send command_not_fount for terminal {terminal_id}");
     senders
@@ -963,6 +1033,7 @@ fn send_command_not_found_to_screen(
             PaneId::Terminal(terminal_id),
             Some(2),
             run_command.clone(),
+            tab_index,
             None,
         ))
         .with_context(err_context)?;
