@@ -16,9 +16,6 @@ use serde::{Deserialize, Serialize};
 use std::fmt::{Display, Error, Formatter};
 use std::path::PathBuf;
 
-use miette::Diagnostic;
-use thiserror::Error as ThisError;
-
 /// Re-exports of common error-handling code.
 pub mod prelude {
     pub use super::FatalError;
@@ -37,39 +34,6 @@ pub trait ErrorInstruction {
     fn error(err: String) -> Self;
 }
 
-#[derive(Debug, ThisError, Diagnostic)]
-#[error("{0}{}", self.show_backtrace())]
-#[diagnostic(help("{}", self.show_help()))]
-struct Panic(String);
-
-impl Panic {
-    // We already capture a backtrace with `anyhow` using the `backtrace` crate in the background.
-    // The advantage is that this is the backtrace of the real errors source (i.e. where we first
-    // encountered the error and turned it into an `anyhow::Error`), whereas the backtrace recorded
-    // here is the backtrace leading to the call to any `panic`ing function. Since now we propagate
-    // errors up before `unwrap`ing them (e.g. in `zellij_server::screen::screen_thread_main`), the
-    // former is what we really want to diagnose.
-    // We still keep the second one around just in case the first backtrace isn't meaningful or
-    // non-existent in the first place (Which really shouldn't happen, but you never know).
-    fn show_backtrace(&self) -> String {
-        if let Ok(var) = std::env::var("RUST_BACKTRACE") {
-            if !var.is_empty() && var != "0" {
-                return format!("\n\nPanic backtrace:\n{:?}", backtrace::Backtrace::new());
-            }
-        }
-        "".into()
-    }
-
-    fn show_help(&self) -> String {
-        r#"If you are seeing this message, it means that something went wrong.
-Please report this error to the github issue.
-(https://github.com/zellij-org/zellij/issues)
-
-Also, if you want to see the backtrace, you can set the `RUST_BACKTRACE` environment variable to `1`.
-"#.into()
-    }
-}
-
 /// Helper trait to easily log error types.
 ///
 /// The `print_error` function takes a closure which takes a `&str` and fares with it as necessary
@@ -215,6 +179,7 @@ pub enum ContextType {
     StdinHandler,
     AsyncTask,
     PtyWrite(PtyWriteContext),
+    BackgroundJob(BackgroundJobContext),
     /// An empty, placeholder call. This should be thought of as representing no call at all.
     /// A call stack representation filled with these is the representation of an empty call stack.
     Empty,
@@ -231,6 +196,7 @@ impl Display for ContextType {
             ContextType::StdinHandler => Some(("stdin_handler_thread:", "AcceptInput".to_string())),
             ContextType::AsyncTask => Some(("stream_terminal_bytes:", "AsyncTask".to_string())),
             ContextType::PtyWrite(c) => Some(("pty_writer_thread:", format!("{:?}", c))),
+            ContextType::BackgroundJob(c) => Some(("background_jobs_thread:", format!("{:?}", c))),
             ContextType::Empty => None,
         } {
             write!(f, "{} {}", left.purple(), right.green())
@@ -283,6 +249,7 @@ pub enum ScreenContext {
     MoveFocusRight,
     MoveFocusRightOrNextTab,
     MovePane,
+    MovePaneBackwards,
     MovePaneDown,
     MovePaneUp,
     MovePaneRight,
@@ -295,6 +262,7 @@ pub enum ScreenContext {
     ScrollDown,
     ScrollDownAt,
     ScrollToBottom,
+    ScrollToTop,
     PageScrollUp,
     PageScrollDown,
     HalfPageScrollUp,
@@ -318,6 +286,7 @@ pub enum ScreenContext {
     SwitchTabPrev,
     CloseTab,
     GoToTab,
+    GoToTabName,
     UpdateTabName,
     UndoRenameTab,
     TerminalResize,
@@ -350,6 +319,11 @@ pub enum ScreenContext {
     SearchToggleCaseSensitivity,
     SearchToggleWholeWord,
     SearchToggleWrap,
+    AddRedPaneFrameColorOverride,
+    ClearPaneFrameColorOverride,
+    PreviousSwapLayout,
+    NextSwapLayout,
+    QueryTabNames,
 }
 
 /// Stack call representations corresponding to the different types of [`PtyInstruction`]s.
@@ -393,6 +367,7 @@ pub enum ClientContext {
     SwitchToMode,
     Connected,
     ActiveClients,
+    Log,
     OwnClientId,
     StartedParsingStdinQuery,
     DoneParsingStdinQuery,
@@ -412,6 +387,7 @@ pub enum ServerContext {
     AttachClient,
     ConnStatus,
     ActiveClients,
+    Log,
 }
 
 #[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
@@ -420,6 +396,12 @@ pub enum PtyWriteContext {
     Exit,
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Serialize, Deserialize)]
+pub enum BackgroundJobContext {
+    DisplayPaneError,
+    Exit,
+}
+
 use thiserror::Error;
 #[derive(Debug, Error)]
 pub enum ZellijError {
@@ -485,8 +467,19 @@ open an issue on GitHub:
         source: anyhow::Error,
     },
 
+    // this is a temporary hack until we're able to merge custom errors from within the various
+    // crates themselves without having to move their payload types here
+    #[error("Cannot resize fixed panes")]
+    CantResizeFixedPanes { pane_ids: Vec<(u32, bool)> }, // bool: 0 => terminal_pane, 1 =>
+    // plugin_pane
+    #[error("Pane size remains unchanged")]
+    PaneSizeUnchanged,
+
     #[error("an error occured")]
     GenericError { source: anyhow::Error },
+
+    #[error("Client {client_id} is too slow to handle incoming messages")]
+    ClientTooSlow { client_id: u16 },
 }
 
 #[cfg(not(target_family = "wasm"))]
@@ -496,13 +489,51 @@ pub use not_wasm::*;
 mod not_wasm {
     use super::*;
     use crate::channels::{SenderWithContext, ASYNCOPENCALLS, OPENCALLS};
-    use miette::{GraphicalReportHandler, GraphicalTheme, Report};
+    use miette::{Diagnostic, GraphicalReportHandler, GraphicalTheme, Report};
     use std::panic::PanicInfo;
+    use thiserror::Error as ThisError;
 
     /// The maximum amount of calls an [`ErrorContext`] will keep track
     /// of in its stack representation. This is a per-thread maximum.
     const MAX_THREAD_CALL_STACK: usize = 6;
 
+    #[derive(Debug, ThisError, Diagnostic)]
+    #[error("{0}{}", self.show_backtrace())]
+    #[diagnostic(help("{}", self.show_help()))]
+    struct Panic(String);
+
+    impl Panic {
+        // We already capture a backtrace with `anyhow` using the `backtrace` crate in the background.
+        // The advantage is that this is the backtrace of the real errors source (i.e. where we first
+        // encountered the error and turned it into an `anyhow::Error`), whereas the backtrace recorded
+        // here is the backtrace leading to the call to any `panic`ing function. Since now we propagate
+        // errors up before `unwrap`ing them (e.g. in `zellij_server::screen::screen_thread_main`), the
+        // former is what we really want to diagnose.
+        // We still keep the second one around just in case the first backtrace isn't meaningful or
+        // non-existent in the first place (Which really shouldn't happen, but you never know).
+        fn show_backtrace(&self) -> String {
+            if let Ok(var) = std::env::var("RUST_BACKTRACE") {
+                if !var.is_empty() && var != "0" {
+                    return format!("\n\nPanic backtrace:\n{:?}", backtrace::Backtrace::new());
+                }
+            }
+            "".into()
+        }
+
+        fn show_help(&self) -> String {
+            format!(
+                "If you are seeing this message, it means that something went wrong.
+
+-> To get additional information, check the log at: {}
+-> To see a backtrace next time, reproduce the error with: RUST_BACKTRACE=1 zellij [...]
+-> To help us fix this, please open an issue: https://github.com/zellij-org/zellij/issues
+
+",
+                crate::consts::ZELLIJ_TMP_LOG_FILE.display().to_string()
+            )
+        }
+    }
+
     /// Custom panic handler/hook. Prints the [`ErrorContext`].
     pub fn handle_panic<T>(info: &PanicInfo<'_>, sender: &SenderWithContext<T>)
     where
