@@ -104,12 +104,24 @@ pub const DEFAULT_LAYOUT: &[u8] = include_bytes!(concat!(
     "assets/layouts/default.kdl"
 ));
 
+pub const DEFAULT_SWAP_LAYOUT: &[u8] = include_bytes!(concat!(
+    env!("CARGO_MANIFEST_DIR"),
+    "/",
+    "assets/layouts/default.swap.kdl"
+));
+
 pub const STRIDER_LAYOUT: &[u8] = include_bytes!(concat!(
     env!("CARGO_MANIFEST_DIR"),
     "/",
     "assets/layouts/strider.kdl"
 ));
 
+pub const STRIDER_SWAP_LAYOUT: &[u8] = include_bytes!(concat!(
+    env!("CARGO_MANIFEST_DIR"),
+    "/",
+    "assets/layouts/strider.swap.kdl"
+));
+
 pub const NO_STATUS_LAYOUT: &[u8] = include_bytes!(concat!(
     env!("CARGO_MANIFEST_DIR"),
     "/",
@@ -122,6 +134,12 @@ pub const COMPACT_BAR_LAYOUT: &[u8] = include_bytes!(concat!(
     "assets/layouts/compact.kdl"
 ));
 
+pub const COMPACT_BAR_SWAP_LAYOUT: &[u8] = include_bytes!(concat!(
+    env!("CARGO_MANIFEST_DIR"),
+    "/",
+    "assets/layouts/compact.swap.kdl"
+));
+
 pub const FISH_EXTRA_COMPLETION: &[u8] = include_bytes!(concat!(
     env!("CARGO_MANIFEST_DIR"),
     "/",
@@ -175,6 +193,18 @@ pub fn dump_specified_layout(layout: &str) -> std::io::Result<()> {
     }
 }
 
+pub fn dump_specified_swap_layout(swap_layout: &str) -> std::io::Result<()> {
+    match swap_layout {
+        "strider" => dump_asset(STRIDER_SWAP_LAYOUT),
+        "default" => dump_asset(DEFAULT_SWAP_LAYOUT),
+        "compact" => dump_asset(COMPACT_BAR_SWAP_LAYOUT),
+        not_found => Err(std::io::Error::new(
+            std::io::ErrorKind::Other,
+            format!("Swap Layout not found for: {}", not_found),
+        )),
+    }
+}
+
 #[cfg(not(target_family = "wasm"))]
 pub fn dump_builtin_plugins(path: &PathBuf) -> Result<()> {
     for (asset_path, bytes) in ASSET_MAP.iter() {
@@ -230,6 +260,10 @@ pub struct Setup {
     #[clap(long, value_parser)]
     pub dump_layout: Option<String>,
 
+    /// Dump the specified swap layout file to stdout
+    #[clap(long, value_parser)]
+    pub dump_swap_layout: Option<String>,
+
     /// Dump the builtin plugins to DIR or "DATA DIR" if unspecified
     #[clap(
         long,
@@ -257,8 +291,8 @@ impl Setup {
     /// file options, superceeding the config file options:
     /// 1. command line options (`zellij options`)
     /// 2. layout options
-    ///    (`layout.yaml` / `zellij --layout`)
-    /// 3. config options (`config.yaml`)
+    ///    (`layout.kdl` / `zellij --layout`)
+    /// 3. config options (`config.kdl`)
     pub fn from_cli_args(cli_args: &CliArgs) -> Result<(Config, Layout, Options), ConfigError> {
         // note that this can potentially exit the process
         Setup::handle_setup_commands(cli_args);
@@ -337,6 +371,11 @@ impl Setup {
             std::process::exit(0);
         }
 
+        if let Some(swap_layout) = &self.dump_swap_layout {
+            dump_specified_swap_layout(swap_layout)?;
+            std::process::exit(0);
+        }
+
         Ok(())
     }
 
